function [x, y ,out] = pdNCGinterface(opts)
%PDNCGINTERFACE interface for pdNCG solver.
%
%  The pdNCG solver is a primal-dual Newton Conjugate Gradients algorithm.
%  The solver implements six formulations in total. Five of them are with 
%  pre-fixed objective function, while one of them is user-defined. 
%  
%  The solver implements:   
%  
%    min tau*||x||_1 + 0.5||Ax-b||_2^2,
%  
%    min tau*||x||_1 + logistic_regression(x),       
%  
%    min tau*||x||_1 + phi(x),
%  
%  where tau is a positive constant given by the user, and phi(x) is 
%  convex and twice differentiable. Additionally, some special 
%  formulations for signal/image reconstruction are implemented:
%  
%    min tau*||W^Tx||_1 + 0.5||Ax-b||_2^2,
%  
%    min tau*||x||_TV + 0.5||Ax-b||_2^2,
%  
%    min tau_W*||W^Tx||_1 + tau_TV*||x||_TV + 0.5||Ax-b||_2^2,
%  
%  where tau_W and tau_TV are positive constants given by the user, W and A 
%  are general matrices, ||x||_TV is the Total-Variation norm and b are the 
%  measurements.
%
%  *** The method works only on vector spaces. Therefore, in case of   ***
%  *** image processing problems the user will have to treat the image ***
%  *** as a vector. Examples are provided in demos folder.             ***
% 
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

if ~isfield(opts, {'problem'})
   error('Possible options for opts.problem: user, ls, lg, wavelet, tv, tv_wavelet.'); 
end

if strcmp(opts.problem,'ls')
  
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_ls);
    
elseif strcmp(opts.problem,'lg')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_lg);
    
elseif strcmp(opts.problem,'wavelet')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_W);
    
elseif strcmp(opts.problem,'tv')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_TV);
    
elseif strcmp(opts.problem,'tv_wavelet')
     
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework_tv_w(opts,@pdNCG_TV_W);
    
elseif strcmp(opts.problem,'user')
    
    % Initialize parameters.
    opts    = checkOptions(opts);
    % Call continuation framework with the solver as an argument.
    [x, y ,out] = cont_framework(opts,@pdNCG_user);
    
else
    error('Possible options for opts.problem: user, ls, lg, wavelet, tv, tv_wavelet.');     
end