function [x, y, out] = pdNCG_lg(opts)
%PDNCG pdNCG solver for the l1-regularized problem
%
% min tau*||x||_1 + phi(X,labels),
%
% where phi(X,labels) is the strongly convex logistic regression function,
% X is the matrix which stores the training samples and y are the labels.

%% Initialize output.
out.itr  = 1; % pdNCG iterations.
out.f = sparse(opts.maxpdNCGitr,1); % Objective values at every iteration.
out.t = sparse(opts.maxpdNCGitr,1); % time passed at every iteration.

%% Initialize data.
stagnation = 0;

[m, ~] = size(opts.X);

% PCG data structure that carries everything the call to the
% preconditioned CG method needs.
PCG = [];

nwtn = zeros(opts.n,1);

x = opts.x0;

D  = 1./sqrt(opts.mu^2 + x.^2);
Dx = D.*x;

y = opts.y0;

% Calculate the approximate Hessian of pseudo-Huber function.
H  = D.*(1 - Dx.*y);
        
tauH = opts.tau.*H;

% Initialize diagonal preconditioner if needed.
if strcmp(opts.PCG_method,'lg_diag_pcg')
    PCG.tauH = tauH;
end

% Calculate gradient at x0.
Xx  = opts.X*x;
yXx = opts.y.*Xx;

t = 1./(1+exp(-yXx));
g_phi = opts.X.'*(opts.y.*(t - 1));

g = opts.tau.*Dx + g_phi;

scale = t.*(1-t);

% Fidelity and regularization functions.
f_phi   = @(q) sum(log(1 + exp(-(opts.y).*(opts.X*q))));
f_huber = @(q) opts.tau*sum(sqrt(opts.mu^2 + q.^2) - opts.mu);
    
%% Print
if opts.verbose >= 1
    fprintf('%5s | %5s    | %5s    | %5s    | %5s | %5s |  %5s  \n',...
        'pdNCG itr', 'a', 'tau', 'mu', '|d|_x/|x|','PCG itr', 'PCG rs');
end

%% Start Newton conjugate gradients.
while 1        
    % Define regularized second-order derivative.
    Hessian = @(q) tauH.*q + opts.X'*(scale.*(opts.X*q)) + opts.reg*q;
    
    % Initialize diagonal preconditioner if needed.
    if strcmp(opts.PCG_method,'lg_diag_pcg')
        PCG.diag_Hes = sum(opts.X.*(sparse(1:m,1:m,scale)*opts.X),1)';
    end
    
    % Calculate Newton direction.  
	[nwtn, rlrs, pcgi, preconditioner] = PCG_wrapper(PCG, Hessian, g, opts);   
    
    % Call user function.
    if isfield(opts, {'user_fcn'}) 
        opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,preconditioner,nwtn);
    end      
    
    normN_scaled = -(nwtn'*g + opts.reg*norm(nwtn)^2)/norm(x);    
        
    dy = H.*nwtn - (y - Dx);
        
    y_tilde = y + dy;
    y = sign(y_tilde).*min(abs(y_tilde),1);
        
    % Calculate stepsize and update variables.        
    [x, a, f_norm1_new] = simple_bt(x, nwtn, f_phi, f_huber, normN_scaled, opts);
    
    % Store objective function and corresponding time.
    out.f(out.itr) = f_norm1_new;
    out.t(out.itr) = toc; 
    
    % The PCG methods needs to know the stepsize.
    PCG.a = a;
    
    % Printing info.
    if opts.verbose >= 1
        fprintf('  %4d    | %1.2e | %1.2e | %1.2e |  %8s | %4d    | %1.2e \n',...
            out.itr, a, opts.tau, opts.mu, num2str(normN_scaled,'%1.2e\n'), pcgi, rlrs);
    end
        
    % Calculate hessian of pseudo-Huber function.
    D  = 1./sqrt(opts.mu^2 + x.^2);
    Dx = D.*x;
    H  = D.*(1 - Dx.*y);
        
    tauH = opts.tau.*H;
    
    % Initialize diagonal preconditioner if needed.
    if strcmp(opts.PCG_method,'lg_diag_pcg')
        PCG.tauH = tauH;
    end
        
    % Calculate first-order derivative of perturbed problem.
    Xx  = opts.X*x;
    yXx = opts.y.*Xx;

    t = 1./(1+exp(-yXx));
    g_phi = opts.X.'*(opts.y.*(t - 1));
    g     = opts.tau.*Dx + g_phi;
    
    scale = t.*(1-t);
    
    if isfield(opts, {'user_stop'}) 
        if opts.user_stop(opts,x,y,@(q) f_huber(q) + f_ls(q),g,...
                          Hessian,nwtn)
            break;
        end
    else
        if normN_scaled <= opts.tol
            out.message = 'Converged';
            break;
        end         
    end
    
    % Maximum iterations reached.
    if out.itr > opts.maxpdNCGitr
        stagnation = 1;
    end
    
    if stagnation == 1
       out.message = 'Maximum iterations of pdNCG reached';
       fprintf('Maximum iterations of pdNCG reached, termination.\n'); 
       fprintf('Possible fixes: \n');
       fprintf('a) increase opts.maxpdNCGitr. Performs more pdNCG iterations.\n');
       fprintf('b) decrease opts.tolpcg. Calculates better directions. \n');
       fprintf('c) increase opts.mxitrpcg. \n');
       break;
    end 
    
    % Update iteration counter.
    out.itr = out.itr + 1;
end
%% End of primal-dual Newton conjugate gradients.

out.f = out.f(1:out.itr);
out.t = out.t(1:out.itr);

% Call user function.
if isfield(opts, {'user_fcn'}) 
    opts.user_fcn(opts,x,y,@(q) f_huber(q) + f_ls(q),g,Hessian,preconditioner,nwtn);
end  